@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2013  Warzone 2100 Project
+	Copyright (C) 2005-2015  Warzone 2100 Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -23,11 +23,10 @@
 
 #include "lib/framework/types.h"
 
-void PlayList_Init(void);
-void PlayList_Quit(void);
+void PlayList_Init();
+void PlayList_Quit();
 bool PlayList_Read(const char *path);
-const char *PlayList_CurrentSong(void);
-const char *PlayList_NextSong(void);
-void playListTest(void);
+const char *PlayList_CurrentSong();
+const char *PlayList_NextSong();
 
 #endif // __INCLUDED_LIB_SOUND_PLAYLIST_H__
